import React, { forwardRef, useState } from 'react';
import PropTypes from 'prop-types';
import { Select, FormControl } from '@material-ui/core';

import { makeStyles } from '@material-ui/core/styles';
import getStyles from './DropDown.styles';

import MenuItem from '../MenuItem/MenuItem';

/**
 * Dropdown component that allows a user to select an option from a pre-defined menu of options.
 * @see https://material-ui.com/demos/selects/
 */
const DropDown = (props) => {
    const {
        contextStyles,
        items,
        disabled,
        testID,
        value,
        onChange,
        renderItem,
        renderValue,
        autoFocus,
    } = props;

    const [
        open,
        setOpen,
    ] = useState(!!autoFocus);

    // Override core styles with context styles, separating MUI styles
    const {
        menuItem: menuItemStyles,
        selectedItem: selectedMenuItemStyles,
        ...muiStyles
    } = getStyles(contextStyles);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate container props
    const containerProps = {
        classes: {
            root: styles.container,
        },
        className: 'c-drop-down__container',
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__drop-down__container`; }

    // Generate select props
    const selectProps = {
        classes: {
            root: styles.dropDown,
            icon: styles.icon,
        },
        className: 'c-drop-down__menu',
        disabled,
        MenuProps: {
            classes: {
                paper: styles.menu,
            },
        },
        onChange,
        renderValue,
        value,
        open,
        onClose: () => setOpen(false),
        onOpen: () => setOpen(true),
    };
    if (testID) { selectProps['data-test-id'] = `${testID}__drop-down__menu`; }

    // Generate menu item props
    const itemProps = {
        className: 'c-drop-down__menu-item',
        contextStyles: {
            container: menuItemStyles,
            selected: selectedMenuItemStyles,
        },
    };
    if (testID) { itemProps.testID = `${testID}__drop-down__menu-item`; }

    // Create menu item with base props used to create this menu instance
    // eslint-disable-next-line react/display-name
    const CustomMenuItem = forwardRef((cProps, ref) => (
        <MenuItem
            ref={ ref }
            { ...itemProps }
            { ...cProps }
            selected={ cProps.text === value } />
    ));

    return (
        <FormControl { ...containerProps }>
            <Select { ...selectProps }>
                { items.length === 0 && (
                    <MenuItem
                        onClick={ () => null }
                        text="(empty)" />
                ) }
                { items.map((item) => renderItem(CustomMenuItem, item)) }
            </Select>
        </FormControl>
    );
};

DropDown.propTypes = {
    /** Auto focus the dropdown menu */
    autoFocus: PropTypes.bool,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,

        /** Styles for the menu */
        menu: PropTypes.object,

        /** Styles for each menu option */
        menuItem: PropTypes.object,

        /** Styles for the Selected Item in the menu */
        selectedItem: PropTypes.object,
    }),

    /** Disable the dropdown */
    disabled: PropTypes.bool,

    /** An array of drop down menu items */
    items: PropTypes.array,

    /**
     * Called when change event occurs on the dropdown
     * @param {Event} event - The change event
     */
    onChange: PropTypes.func.isRequired,

    /**
     * Called to render the menu items
     * @param {Component} MenuItem - The default menu item component
     * @param {any} Item - Item that needs to be rendered
     */
    renderItem: PropTypes.func.isRequired,

    /**
     * Function that is used to render a custom selected option
     * @param {any} value - The current selected item.
     */
    renderValue: PropTypes.func.isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Selected Item value */
    value: PropTypes.string,
};

DropDown.defaultProps = {
    contextStyles: {},
    value: '',
    items: [],
};

export default DropDown;
