import React from 'react';
import MuiIcon from '@material-ui/core/Icon';
import PropTypes from 'prop-types';

import { makeStyles } from '@material-ui/core/styles';
import getStyles from './Icon.styles';

/** Default icon size */
const ICON_SIZE_DEFAULT = 'default';

const ICON_SIZE = {
    DEFAULT: 'default',
    INHERIT: 'inherit',
    LARGE: 'large',
    SMALL: 'small',
};

/**
 * Icon component allows a user to view a non-image visual representation of an object or action.
 * @see https://material-ui.com/style/icons/
 */
const Icon = (props) => {
    const {
        contextStyles,
        iconName,
        iconSize,
        testID,
        title,
    } = props;

    // Override core styles with context styles, separating MUI styles
    const {
        container: containerStyles,
        ...muiStyles
    } = getStyles(contextStyles);

    // Create dynamic class names and injected DOM styles for MUI component
    // * makeStyles returns a function
    const styles = makeStyles(muiStyles)();

    // Generate container props
    const containerProps = {
        className: 'c-icon__container',
        style: containerStyles, // non-MUI styles
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__icon__container`; }

    // Generate icon props
    const iconProps = {
        classes: {
            root: styles.icon, // MUI styles
        },
        className: 'c-icon__icon',
        fontSize: iconSize,
        title: title || iconName,
    };
    if (testID) { iconProps['data-test-id'] = `${testID}__icon__icon`; }

    return (
        <span { ...containerProps }>
            <MuiIcon { ...iconProps }>
                { iconName }
            </MuiIcon>
        </span>
    );
};

Icon.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the icon */
        container: PropTypes.object,

        /** Styles for a icon */
        icon: PropTypes.object,
    }),

    /** Name of a UI icon, or an icon element */
    iconName: PropTypes.string.isRequired,

    /** Size of the icon to be rendered */
    iconSize: PropTypes.oneOf([
        ICON_SIZE.DEFAULT,
        ICON_SIZE.INHERIT,
        ICON_SIZE.LARGE,
        ICON_SIZE.SMALL,
    ]),

    /** Id used for testing */
    testID: PropTypes.string,

    /**
     * Description string.
     * Browsers usually render this as a tooltip on hover, whereas screen
     * readers will use it for aural feedback.
     * By default, this is set to the icon's name for accessibility.
     */
    title: PropTypes.string,
};

Icon.defaultProps = {
    contextStyles: {},
    iconSize: ICON_SIZE_DEFAULT,
};

export default Icon;
export {
    ICON_SIZE,
};
