import React, { useState } from 'react';
import PropTypes from 'prop-types';
import ActivityIndicator from '../ActivityIndicator/ActivityIndicator';

import getStyles from './Image.styles';

/**
 * Image Component Class, that will render the supplied image url.
 * While Image is loading it will show a loader.
 */
const Image = (props) => {
    const {
        contextStyles,
        imageSrc,
        isDisabled,
        onClick,
        showLoadingIndicator,
        testID,
    } = props;
    const [
        isImageLoaded,
        setImageLoaded,
    ] = useState(false);
    const showIndicator = showLoadingIndicator && !isImageLoaded;

    const styles = getStyles(contextStyles, isDisabled);

    // Generate container props
    const containerProps = {
        className: 'c-image__container',
        style: styles.container,
    };
    if (testID) { containerProps['data-test-id'] = `${testID}__image__container`; }

    // Generate indicator props
    const indicatorProps = {
        centered: true,
        contextStyles: {
            container: {},
            indicator: {},
        },
    };
    if (testID) { indicatorProps.testID = `${testID}__image__indicator`; }

    // Generate image props
    const imageProps = {
        className: 'c-image__image',
        onLoad: () => { setImageLoaded(true); },
        src: imageSrc,
        style: styles.image,
    };
    if (!isDisabled) { imageProps.onClick = onClick; }
    if (testID) { imageProps['data-test-id'] = `${testID}__image__image`; }

    return (
        <div
            style={ styles.container }>
            { showIndicator &&
                <ActivityIndicator { ...indicatorProps } />
            }
            <img
                alt={ imageSrc }
                { ...imageProps } />
        </div>
    );
};

Image.propTypes = {
    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the image */
        container: PropTypes.object,

        /** Styles for the image */
        image: PropTypes.object,

        /** Styles for the loading indicator */
        loadingIndicator: PropTypes.object,
    }),

    /** The image source (either a remote URL or a local file resource) */
    imageSrc: PropTypes.string.isRequired,

    /** Disables image touch */
    isDisabled: PropTypes.bool,

    /**
     * Called when the image was pressed
     * @param {Function} props - The component props
     * @param {Object} event - The native press event
     */
    onClick: PropTypes.func,

    /** Controls the loading indicator visibility */
    showLoadingIndicator: PropTypes.bool,

    /** Id used for testing */
    testID: PropTypes.string,
};

Image.defaultProps = {
    contextStyles: {},
    showLoadingIndicator: true,
};

export default Image;
